BagWindow = class( Turbine.UI.Lotro.Window );

function BagWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	-- Padding used to position the bag initially from the bottom
	-- right of the screen.
	local edgePadding = 50;

	self.ListBoxes = {}
	self.ItemCounts = {}
	self.items = {}

	self.callbacks = {}

	local player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.backpack = player:GetBackpack();


	self:SetPosition( Settings.BagX, Settings.BagY );
	--self:SetBackColor( Turbine.UI.Color() );
	self:SetSize( Settings.BagWidth, Settings.BagHeight);
	self:SetText( "Prime Bag" );
    self:SetOpacity( 1);
	self:SetVisible(false)
	self:SetResizable(true)

	self.SizeChanged = function()
		self:PerformLayout()
	end

	
	self.resizer = Resizer(self)

	-- Handle bag shortcut
	self:SetWantsKeyEvents( true );
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			self:SetVisible( false ) 
		end

		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			self:SetVisible( not self:IsVisible() ) 
			if self:IsVisible() then 
				self:Focus() 
				--self:SetZOrder(100)
			end
		end
	end

	-- Set up list box holder
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );

	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	self.itemListBox:SetAllowDrop( true );

	-- Handle any changes to the backpack

	self.callbacks["SizeChanged"] = AddCallback(self.backpack, "SizeChanged", function(sender, args) self:BuildBag() end)

	self.callbacks["ItemAdded"] = AddCallback(self.backpack, "ItemAdded", function(sender, args) 
		local item = sender:GetItem(args.Index)
		local category = ItemLUT[item:GetItemInfo():GetCategory()]

		self.ListBoxes[self.items[args.Index]:GetCategory()]:RemoveItem(self.items[args.Index])

		if self.ListBoxes[category] == nil then category = "Miscellaneous" end

		self.items[args.Index]:SetCategory(category)
		self.items[args.Index]:SetItem(item) 
		self.ListBoxes[category]:AddItem(self.items[args.Index])
		
	end)

	self.callbacks["ItemRemoved"] = AddCallback(self.backpack, "ItemRemoved", function(sender, args) 

		self.ListBoxes[self.items[args.Index]:GetCategory()]:RemoveItem(self.items[args.Index])
		self.items[args.Index]:SetItem(nil) 
		self.items[args.Index]:SetCategory("Empty Slot")
		self.ListBoxes["Empty Slot"]:AddItem(self.items[args.Index])
	end)

	self.callbacks["ItemMoved"] = AddCallback(self.backpack, "ItemMoved", function(sender, args)
		local categoryOld = self.items[args.OldIndex]:GetCategory()
		local categoryNew = self.items[args.NewIndex]:GetCategory()

		self.ListBoxes[categoryOld]:RemoveItem(self.items[args.OldIndex])
		self.ListBoxes[categoryOld]:AddItem(self.items[args.OldIndex])

		self.ListBoxes[categoryNew]:RemoveItem(self.items[args.NewIndex])
		self.ListBoxes[categoryNew]:AddItem(self.items[args.NewIndex])

		self.items[args.NewIndex]:SetCategory(categoryNew)
		self.items[args.OldIndex]:SetCategory(categoryOld)

		--self.items[args.OldIndex]:SetItem(sender:GetItem(args.OldIndex))
		--self.items[args.NewIndex]:SetItem(sender:GetItem(args.NewIndex))
	end)

	--[[
	self.VisibleChanged = function()
		if self:IsVisible() then self:BuildBag() end
	end
	--]]

	
	self.Walletbar = WalletBar(self)

	-- Refresh button

	--0x4100024F
	--0x41000250
	--0x41000251
	--0x41000252
	self.ReloadButton = Turbine.UI.Control()
	self.ReloadButton:SetParent(self)
	self.ReloadButton:SetSize(20, 20)
	self.ReloadButton:SetPosition(self:GetWidth() - 50, 20)
	self.ReloadButton:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.ReloadButton:SetBackground(0x41000250)

	self.ReloadButton.MouseEnter = function()
		self.ReloadButton:SetBackground(0x4100024F)
	end
	
	self.ReloadButton.MouseLeave = function()
		self.ReloadButton:SetBackground(0x41000250)
	end

	self.ReloadButton.MouseClick = function()
		self:BuildBag()
		self.Walletbar:Refresh()
	end

	self:BuildBag()
end

function BagWindow:BuildBag()
	local backpackSize = self.backpack:GetSize();

	-- Sum itemcategories
	self.ItemCounts = {}
	for i = 1, backpackSize, 1 do
		if self.backpack:GetItem( i ) then
			if ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()] then
				if self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] then 
					self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] = self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] + 1
				else
					self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] = 1
				end
			else
				if self.ItemCounts[0] then 
					self.ItemCounts[0] = self.ItemCounts[0] + 1
				else
					self.ItemCounts[0] = 1
				end
			end
		else
			if self.ItemCounts[-2] then
				self.ItemCounts[-2] = self.ItemCounts[-2] + 1
			else
				self.ItemCounts[-2] = 1
			end
		end
	end	

	-- Handle undefined items
	self.ItemCounts[-1] = 0
	for key, value in pairs(self.ItemCounts) do
		if value < 5 and key ~= -2 then self.ItemCounts[-1] = self.ItemCounts[-1] + value end
	end

	for key, value in pairs(self.ListBoxes) do 
		value:ClearItems()
	end

	self.itemListBox:ClearItems() 

	-- Setup category list boxes
	--[[
	local MiscCat = false
	for key, value in spairs(self.ItemCounts, function(t,a,b) return t[a] > t[b] end) do
		ChatOutput(key .. value)
		if value > 4 then
			self.ListBoxes[key] = CategoryListBox(value, key, self)
		else
			MiscCat = true
		end
	end
	--]]
	
	-- [[
	for key, _ in pairs(ItemGroups) do
		if self.ItemCounts[key] and self.ItemCounts[key] > 0 then 
			if self.ItemCounts[key] < 5 then
				MiscCat = true
			else
				self.ListBoxes[key] = CategoryListBox(self.ItemCounts[key], key, self)
			end
		
		end
	end
	--]]

	-- Handle outlier categories
	if MiscCat then self.ListBoxes["Miscellaneous"] = CategoryListBox(self.ItemCounts[-1], "Miscellaneous", self) end
	if self.ItemCounts[0] and self.ItemCounts[0] > 0 then self.ListBoxes["Undefined"] = CategoryListBox(self.ItemCounts[0], "Undefined", self) end
	self.ListBoxes["Empty Slot"] = EmptyListBox(self.ItemCounts[-2], "Empty Slot", self)

	-- Setup the bag
	for i = 1, backpackSize, 1 do

		-- Delete old data if there is some
		if self.items[i] then self.items[i] = nil --[[self.items[i]:SetParent(nil)]] end
		
		-- Create the bag slot
		local bagSlot = BagItem( self.backpack:GetItem(i), i);
		self.items[i] = bagSlot
			
		-- Handle addition to correct category
		if self.backpack:GetItem(i) then 

			--ChatOutput(self.backpack:GetItem( i ):GetItemInfo():GetName() .. self.backpack:GetItem( i ):GetItemInfo():GetCategory())

			local Cat = ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]
			if not(Cat) then 
				Cat = "Undefined" 
			elseif self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] and self.ItemCounts[ItemLUT[self.backpack:GetItem( i ):GetItemInfo():GetCategory()]] < 5 then 
				Cat = "Miscellaneous" 
			end

			bagSlot:SetCategory(Cat)
			self.ListBoxes[Cat]:AddItem(bagSlot)
		else
			bagSlot:SetCategory("Empty Slot")
			self.ListBoxes["Empty Slot"]:AddItem(bagSlot)
		end
	end

	for key, value in spairs(self.ListBoxes, function(t,a,b) return t[a]:GetItemCount() > t[b]:GetItemCount() end) do
		self.itemListBox:AddItem( value );
	end

	self:PerformLayout();
end

function BagWindow:PerformLayout()
	self:Layout( { } )
end

function BagWindow:Layout( args )
	local width, height = self:GetSize();
	
	local itemWidth = 32;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = width - 40;
	local listHeight = height - 50;
	local itemsPerRow = math.floor(width / (35 * 5));

	self.itemListBox:SetPosition( 15, 35 );
	self.itemListBox:SetSize( listWidth, listHeight - 20 );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	--self.itemListBox:SetBackColor(Turbine.UI.Color(0,0,1))
	
	self.itemListBoxScrollBar:SetPosition( width - 25, 35 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight );

	self.Walletbar:SetPosition(15, 35 + listHeight - 20)

	self.ReloadButton:SetPosition(self:GetWidth() - 50, 20)
	
end

function BagWindow:GetBackpack()
	return self.backpack
end

function BagWindow:RemoveCallbacks()
	for key, value in pairs(self.callbacks) do
		RemoveCallback(self.backpack, key, value)
	end

	self.Walletbar:RemoveCallbacks()
end

