AlertWindow = class(Turbine.UI.Window)
function AlertWindow:Constructor()
    local width = Settings.AWidth
    local height = Settings.AHeight
    local x = Settings.AX
    local y = Settings.AY
    self.Alerts = {}
    self.resort = false
    self.AlertHeight = Settings.ABaseHeight

    local MaxItems = math.floor(height / Settings.ABaseHeight)

    Turbine.UI.Window.Constructor(self);
    self:SetSize(width,height);
    self:SetPosition(x,y);
    self:SetZOrder(1);

    self:SetWantsKeyEvents(false)
    self:SetMouseVisible(false)
    self:SetVisible(true)
    --self:SetWantsUpdates(true)
    Updater:Subscribe(self)

    self.Clear = function()
        --self.messageBox:SetText("")
        --cooldownTimer = 0
        --self.imageBox1:SetBackground("")
        --self.imageBox2:SetBackground("")
    end

    self.ReUpdateAlertPositions = function()

        local counter = 1
        for _,v in spairs(self.Alerts, function(t,a,b) return t[b]:GetDuration() > t[a]:GetDuration() end) do
            v:SetPosition(0, MaxItems * (self.AlertHeight) - (counter * self.AlertHeight)) 
            counter = counter + 1
        end

        self.resort = false
    end

    local moveBox = MoveBox(self, width, height, "Alert Window")
    
    self.ToggleMove = function()
        moveBox:ToggleMove()
    end
end

function AlertWindow:Message(duration, message, icon, cdVisible)
    self.Alerts[#self.Alerts + 1] = Alert(self, duration, message, icon, cdVisible)
    self.resort = true
    --Alerts[5] = Alert(self, 10, "Test", PuddleIconSmall, true)
    ChatOutput(message)
end

function AlertWindow:ExecuteUpdate(delta)
    for key, value in pairs(self.Alerts) do
        if value ~= nil then
            value:UpdateText(delta)
            if value:GetDuration() <= 0 then
                value:Destroy()
                self.Alerts[key] = nil
                self.resort = true
            end
        end
    end
    if self.resort then
        self.ReUpdateAlertPositions()
    end
end

Alert = class(Turbine.UI.Window)
function Alert:Constructor(parent, duration, message, icon, cdVisible)
    --ChatOutput("Printing:")
    --ChatOutput("Message: " .. message)
    --ChatOutput("Duration: " .. duration)
    --ChatOutput("Icon: " .. icon)
    --ChatOutput("CdVisible: " .. cdVisible)

    Turbine.UI.Window.Constructor(self)
    self.timer = duration
    self.showTimers = cdVisible
    self.oldtimer = duration
    self.colourEvent = 2
    self.showLeft = Settings.AL
    self.showRight = Settings.AR
    self.colourChange = true
    self:SetParent(parent)
    self:SetSize(Settings.AWidth, Settings.ABaseHeight)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetVisible(true)

    local messageBox = Turbine.UI.Label()
    messageBox:SetParent(self)
    messageBox:SetSize(600, Settings.ABaseHeight)
    messageBox:SetPosition(Settings.ABaseHeight + 5,0)
    messageBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    messageBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    messageBox:SetMouseVisible(false)
    messageBox:SetVisible(true)
    messageBox:SetMarkupEnabled(true)
    messageBox:SetOutlineColor(OutlineColor)
    messageBox:SetFont(Settings.AFont)
    messageBox:SetText(message)
    messageBox:SetMultiline(false)
    --self.messageBox:SetBackColor(Turbine.UI.Color(0,0,0))

    self.imageBox1 = Turbine.UI.Label()
    self.imageBox1:SetParent(self)
    self.imageBox1:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    self.imageBox1:SetPosition(0, 0)
    self.imageBox1:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.imageBox1:SetFont(Settings.AFont)
    self.imageBox1:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.imageBox1:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.imageBox1:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.imageBox1:SetMouseVisible(false)

    self.imageBox2 = Turbine.UI.Label()
    self.imageBox2:SetParent(self)
    self.imageBox2:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    self.imageBox2:SetPosition(Settings.AWidth - Settings.ABaseHeight, 0)
    self.imageBox2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.imageBox2:SetFont(Settings.AFont)
    self.imageBox2:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.imageBox2:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.imageBox2:SetMouseVisible(false)

    if icon ~= nil then
        if Settings.AL then
            self.imageBox1:SetBackground(icon)
            self.imageBox1:SetVisible(true)
        end
        if Settings.AR then
            self.imageBox2:SetBackground(icon)
            self.imageBox2:SetVisible(true)
        end
      else
        self.imageBox1:SetBackground("")
        self.imageBox1:SetVisible(true)
        self.imageBox2:SetBackground("")
        self.imageBox2:SetVisible(true)
      end
    
      if cdVisible then
        if Settings.AL then
            self.imageBox1:SetText(duration)
        end
        if Settings.AR then
            self.imageBox2:SetText(duration)
        end
      else 
        self.imageBox1:SetText("")
        self.imageBox2:SetText("")
      end

    self.Destroy = function()
        self:SetVisible(false)
        self:SetParent(nil)
    end

    self.GetDuration = function()
        return(self.timer)
    end
end

function Alert:UpdateText(delta)
    self.timer = self.timer - delta

    if self.timer < 1 then
        self:SetOpacity(self.timer + 0.3)
    end
    
    if self.oldtimer - self.timer > 0.05 then
        if self.showTimers and self.showLeft then
            if self.timer <= self.colourEvent then
                self.imageBox1:SetText(Round(self.timer,1))
                if self.colourChange then
                    self.imageBox1:SetForeColor(Turbine.UI.Color(1, 0.75, 0))
                    --self.imageBox1:SetOutlineColor(Turbine.UI.Color(1, 0, 0))
                    self.colourChange = false
                end
                --self.colourEvent = -1
            else
                self.imageBox1:SetText(math.floor(self.timer))
            end
            --self.imageBox1:SetText(math.floor(self.timer))
            if self.showRight then
                self.imageBox2:SetText(math.floor(self.timer))
            end
        end
        self.oldtimer = self.timer
    end
end