CaptainMotivated = class(Turbine.UI.Window)
function CaptainMotivated:Constructor()
    Turbine.UI.Window.Constructor(self)
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    

    self.width = 50
    self.height = 50

    self.callbacks = nil
    self.skillItem = nil

    self.effectID = 0

    self.isMotivated = false

    if Settings["ItemPositions"]["CaptainMotivated"] == nil then
        Settings["ItemPositions"]["CaptainMotivated"] = {}
        Settings["ItemPositions"]["CaptainMotivated"].X = math.floor(Turbine.UI.Display.GetWidth() / 2 - self.width / 2) 
        Settings["ItemPositions"]["CaptainMotivated"].Y = 400
    end
    

    self:SetPosition(Settings["ItemPositions"]["CaptainMotivated"].X, Settings["ItemPositions"]["CaptainMotivated"].Y)
    self:SetSize(self.width, self.height)
    self:SetVisible(true)

    self.banner = Turbine.UI.Control()
    self.banner:SetParent(self)
    self.banner:SetMouseVisible(false)
    self.banner:SetVisible(false)
    self.banner:SetSize(32, 32)
    self.banner:SetBackground(1090553780)
    --self.banner:SetStretchMode(1)
    --self.banner:SetSize(50, 50)

    self.highlightFrame = ClassSpecificTools.HighlightFrame(self.banner)
    
    self.MoveBox = Turbine.UI.Window()
    self.MoveBox:SetParent(self)
    self.MoveBox:SetSize(self.width, self.height)
    self.MoveBox:SetVisible(true)

    self.Moving = false

    self.MoveBox.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.MoveBox.MouseUp = function()
        self.Moving = false
    end

    self.MoveBox.MouseMove = function(sender,args)
        if self.Moving then
        local newLeft = self:GetLeft() - (self.MoveX - args.X)
        local newTop = self:GetTop() - (self.MoveY - args.Y)
        self:SetPosition(newLeft,newTop)
        end
    end

    self.skills = self.player:GetTrainedSkills()
    self.effectList = self.player:GetEffects()
    self:SetCallbacks()
end

function CaptainMotivated:SetCallbacks()

    for i = 1, self.player:GetEffects():GetCount(), 1 do
        local effect = self.player:GetEffects():Get(i)
        if effect == "Motivated" then
            self.isMotivated = true
            self.effectID = effect:GetID()
        end
    end

    for i = 1, self.skills:GetCount(), 1 do
        local item = self.skills:GetItem(i)--:GetSkillInfo()
        if item:GetSkillInfo():GetName() == "Improved Motivating Speech" then
            self.skillItem = item
            if item:GetResetTime() == -1 and not(self.isMotivated) then self:Activate(true) end
        end
    end

    self.effectCallbacks = AddCallback(self.effectList, "EffectAdded", 
    function(sender, args)
        local effect = sender:Get(args.Index) 

        if effect:GetName() == "Motivated" then
            self.isMotivated = true
            self.effectID = effect:GetID()
            self:Activate(false)
        end
    end)

    self.removedEffectCallbacks = AddCallback(self.effectList, "EffectRemoved", 
    function(sender, args)
        local effect = args.Effect

        if effect then

            if effect:GetName() == "Motivated" and effect:GetID() == self.effectID then 
                self.isMotivated = false

                if self.skillItem:GetResetTime() == -1 then 
                    self:Activate(true) 
                else
                    self.timer = self.skillItem:GetResetTime() - Turbine.Engine.GetGameTime()
                    Updater:Subscribe(self)
                end

            end
        end
    end)

end

function CaptainMotivated:ExecuteUpdate(delta)
    self.timer = self.timer - delta
    if self.timer < 0 then
        Updater:UnSubscribe(self)
        if not(self.isMotivated) then
            self:Activate(true)
        end
    end
end

function CaptainMotivated:Activate(value)
    self:SetVisible(value)
    self.banner:SetVisible(value)
    self.highlightFrame:Toggle(value)
end

function CaptainMotivated:RemoveCallbacks()
    --if self.skillCallback then RemoveCallback(self.skillItem, "ResetTimeChanged", self.skillCallback) end   
    if self.effectCallbacks then RemoveCallback(self.effectList, "EffectAdded", self.effectCallbacks) end 
    if self.removedEffectCallbacks then RemoveCallback(self.effectList, "EffectRemoved", self.removedEffectCallbacks) end 
end

function CaptainMotivated:Unload()
    self:RemoveCallbacks()
    Settings["ItemPositions"]["CaptainMotivated"].X, Settings["ItemPositions"]["CaptainMotivated"].Y = self:GetPosition()
    self:SetVisible(false)
    Updater:UnSubscribe(self)
    self:SetParent(nil)
end