DynamicSkillBar = class(Turbine.UI.Control)
function DynamicSkillBar:Constructor(parent, x, y, width, height, labelSize, sortMethod, hide)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(width, height)
    self:SetMouseVisible(false)
    --self:SetBackColor(GetRandomColour())

    self.width = width

    self.barHeight = 0
    self.barWidth = 0

    self.xCounts = {}

    self.width = width
    self.labelSize = labelSize

    self.skills = {}
    self.classLabels = {}
    self.activeClassLabels = {}
    self.timerLabels = {}
    self.skillLUT = {}
    self.yIndex = {}

    self.activeTimerCount = 0

    self.sortTable = {}

    self.activeCount = 0

    self.hide = not(hide)

    -- Make efficient sorting
    if sortMethod == 1 then -- Left
        self.SortLabels = function()
            local counter = 0
            for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                self.classLabels[key]:SetPosition(counter * self.labelSize, 0)
                counter = counter + 1
            end
        end
    elseif sortMethod == 2 then -- Center
        self.SortLabels = function()
            local counter = 0
            for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                self.classLabels[key]:SetPosition(self.width / 2 - self.labelSize / 2 * self.activeCount + counter * self.labelSize, 0)
                counter = counter + 1
            end
        end
    elseif sortMethod == 3 then -- Right
        self.SortLabels = function()
            local counter = 0
            for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                self.classLabels[key]:SetPosition(self.width - (counter + 1) * self.labelSize, 0)
                counter = counter + 1
            end
        end
    elseif sortMethod == 4 then
        self.SortLabels = function()

        end
    end

end

function DynamicSkillBar:AddLabel(skill, label, priority, skillObject, y)
    self.sortTable[skill] = priority

    self.classLabels[skill] = label
    self.skillLUT[skill] = skillObject

    if y == nil then y = 1 end

    if not(self.yIndex[y]) then self.yIndex[y] = {} end

    self.yIndex[y][skill] = true

    -- Update counts
    if self.xCounts[y] then
        self.xCounts[y] = self.xCounts[y] + 1
    else
        self.xCounts[y] = 1
    end

    if self.barHeight < y then 
        self.barHeight = y 
        self:SetHeight(y * 32)
    end
end

function DynamicSkillBar:ExpandHeight(y)
    self:SetHeight(y)

    if y / 32 > self.barHeight then
        self.barHeight = y / 32
    end
end

function DynamicSkillBar:MaxHeight()
    return self.barHeight
end

function DynamicSkillBar:SetStaticPositions()
    for yIndex, skills in pairs(self.yIndex) do
        local padding = self.width / 2 - self.xCounts[yIndex] * 16
        local positionCounter = 0
        for skillName, _ in spairs(skills, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
            self.classLabels[skillName]:SetPosition(padding + positionCounter * 32, (yIndex - 1) * 32)
            positionCounter = positionCounter + 1
        end
    end
end

function DynamicSkillBar:MaxWidth()
    local maxWidth = 0 
    for key, value in pairs(self.xCounts) do
        if value > maxWidth then maxWidth = value end
    end
    return maxWidth
end

function DynamicSkillBar:ToggleLabel(skill, value, time, icon)

    -- Set visible
    if value then 
       
        -- Check if already visible
        if self.activeClassLabels[skill] then
            -- Correct timer and return
            if time then self:SetLabelTimer(skill, time) end
            return
        end

        -- Otherwise set visible and update book keeping
        if icon then self.classLabels[skill]:AddBackground(icon) end
        self.activeClassLabels[skill] = true

        self.classLabels[skill]:ToggleVisible(true)
        self.activeCount = self.activeCount + 1
        if time then self:SetLabelTimer(skill, time) end

    else -- Disable label

        -- Check if already disabled
        if self.activeClassLabels[skill] == nil then 
            return 
        end

        --ChatOutput(skill .. "Disable")
        self.classLabels[skill]:ToggleVisible(self.hide)
        self.activeCount = self.activeCount - 1
        self.activeClassLabels[skill] = nil

        if self.timerLabels[skill] then
            self.activeTimerCount = self.activeTimerCount - 1
            self.timerLabels[skill] = nil
            if self.activeTimerCount <= 0 then
                Updater:UnSubscribe(self)
            end
        end
    end

    self:SortLabels()

    --[[
    if self.activeClassLabels[skill] ~= value then
        self.classLabels[skill]:ToggleVisible(value)
        if value then
            self.activeCount = self.activeCount + 1
            self.activeClassLabels[skill] = value
        else
            self.activeCount = self.activeCount - 1 
            self.activeClassLabels[skill] = nil
        end
    end

    if time == true and self.skillLUT[skill]:GetResetTime() > 0 then
        self:SetLabelTimer(skill, self.skillLUT[skill]:GetResetTime() - Turbine.Engine.GetGameTime(), self.skillLUT[skill]:GetCooldown())
    elseif time and time ~= true and time > 0 then
        self:SetLabelTimer(skill, time)
    end
    --]]
end

function DynamicSkillBar:SetLabelTimer(skill, time, maxTime)

    if time <= 0 then return end

    if not(self.timerLabels[skill]) then
        self.activeTimerCount = self.activeTimerCount + 1
    end

    self.classLabels[skill]:SetTimer(time, maxTime)
    self.activeClassLabels[skill] = true
    self.timerLabels[skill] = true

    Updater:Subscribe(self)
end

function DynamicSkillBar:ExecuteUpdate(delta)
    for key, value in pairs(self.timerLabels) do
        if self.classLabels[key]:ExecuteUpdate(delta) then
            self:ToggleLabel(key, nil)
            --if not(self.classLabels[key]:IsVisible()) then self:ToggleLabel(key) end
        end
    end
    --ChatOutput(self.activeTimerCount)
    --ChatOutput(self.activeCount)
    --ChatOutput(self.activeTimerCount)
    --ChatOutput("--")

    if self.activeTimerCount <= 0 then
        Updater:UnSubscribe(self)
    end
end

function DynamicSkillBar:Unload()

    for key, value in pairs(self.classLabels) do
        value:Unload()
    end
    
    self.sortTable = {}
    self.skillLUT = {}
    self.yIndex = {}
    self.skills = {}
    self.classLabels = {}
    self.activeClassLabels = {}
    self.timerLabels = {}
    self.skillLUT = {}
    self.sortTable = {}
    self.xCounts = {}
    self.activeTimerCount = 0
    self.activeCount = 0
    self.barHeight = 0
    self.barWidth = 0
end