PlayerEffectEvents = class(Turbine.UI.Control)
function PlayerEffectEvents:Constructor()
  Turbine.UI.Control.Constructor(self)
  --self.E = E
  self.AllEffectsAdded = E.AllEffectsAdded
  self.SelfEffectsLUT = SelfEffectsLUT
  self.EffectsAdded = E.EffectsAdded
  self.TEffects = TEffects
  self.AllPlayerEffects = E.AllPlayerEffects
  self.PlayerEffectsTable = E.PlayerEffects
  self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()
  self.PlayerEffects = self.Player:GetEffects()

  self.callbacks = {}

  -- Removed effect for buff bars
  self.PlayerEffectRemoved = function()
    self.callbacks["EffectRemoved"] = AddCallback(self.PlayerEffects, "EffectRemoved", function(sender, args)
      if args.Effect then 
        local tempID = args.Effect:GetID()
        for _, value in pairs(BuffWindowObjects) do
          value:RemoveTimer(tempID)
        end
      end
    end)
  end

  self.PlayerEffectRemoved()

  -- Begin Alert window effect handling
  self.PlayerAlertWindowHandling = function(effect)
    local AlertTemp = self.AllEffectsAdded[effect:GetName()]
    if AlertTemp then

      if AlertTemp[5] then Sounds:PlaySoundEffect(AlertTemp[5]) end
      if AlertTemp[3] == true then AlertTemp[3] = effect:GetIcon() end
      messageWindow:Message(AlertTemp[1], AlertTemp[2] .. "<rgb=#FF0000>" .. self.Player:GetName() .. "</rgb>" , AlertTemp[3], AlertTemp[4])

    end
  end
  -- End Alert window effect handling

  -- Begin Effect Alert handling
  self.PlayerEffectWindowHandling = function(effect)
    local EffectTemp = self.EffectsAdded[effect:GetName()]
    if EffectTemp then
      if EffectTemp[5] then Sounds:PlaySoundEffect(EffectTemp[5]) end
      effectsWindow:Message(EffectTemp[1], EffectTemp[2], EffectTemp[3], EffectTemp[4], effect:GetName())
    end
  end
  -- End Effect Alert handling

  -- Begin Player Timer Handling
  self.PlayerTimerHandling = function(effect)
    local TimerTemp = self.TEffects[effect:GetName()]

    if TimerTemp == nil then return end

    local message = nil
    if TimerTemp[2] == "<name>" then 
      message = Player:GetName() 
    else 
      message = TimerTemp[2] 
    end

    local effectInput = nil
    if TimerTemp[4] then effectInput = effect end

    if TimerTemp then skillCounters:NewTimer(TimerTemp[1], message, TimerTemp[3], effect) end
  end
  -- End Player Timer Handling

  -- Begin Player Effect Timer Handling
  self.PlayerEffectTimerHandling = function(effect)
    -- Global effects handling
    local AllPlayerEffectsTemp = self.AllPlayerEffects[effect:GetName()]
    if AllPlayerEffectsTemp then
      local EffectDuration = AllPlayerEffectsTemp[4]
      if effect:GetDuration() and EffectDuration and effect:GetDuration() < 999 then
        EffectDuration = effect:GetDuration()
      elseif EffectDuration == nil then
        EffectDuration = 1
      end
      if EffectDuration then
        activeTimers:NewTimer(AllPlayerEffectsTemp[1], effect:GetName(), AllPlayerEffectsTemp[2], AllPlayerEffectsTemp[3], EffectDuration, AllPlayerEffectsTemp[5], effect:GetIcon(), self.Player:GetName())
      end
      skillTimers:NewTimer(AllPlayerEffectsTemp[1], effect:GetName(), AllPlayerEffectsTemp[2], AllPlayerEffectsTemp[3], EffectDuration, AllPlayerEffectsTemp[5], effect:GetIcon(), self.Player:GetName())
    end

    -- Class specific effects handling
    if self.PlayerEffectsTable[Player:GetClass()] then
      local ClassSpecificEffects = self.PlayerEffectsTable[Player:GetClass()][effect:GetName()]
      if ClassSpecificEffects then
        local EffectDuration = ClassSpecificEffects[4]
        if effect:GetDuration()  and EffectDuration and effect:GetDuration() < 999 then
          EffectDuration = effect:GetDuration()
        elseif EffectDuration == nil then
          EffectDuration = 1
        end
        if EffectDuration then
          activeTimers:NewTimer(ClassSpecificEffects[1], effect:GetName(), ClassSpecificEffects[2], ClassSpecificEffects[3], EffectDuration, ClassSpecificEffects[5], effect:GetIcon(), self.Player:GetName())
        end
        skillTimers:NewTimer(ClassSpecificEffects[1], effect:GetName(), ClassSpecificEffects[2], ClassSpecificEffects[3], EffectDuration, ClassSpecificEffects[5], effect:GetIcon(), self.Player:GetName())
      end
    end
  end
  -- End Player Effect Timer Handling

  -- Begin Buff Windows Effect Handling
  self.PlayerBuffWindowHandling = function(effect)
    for _, value in pairs(BuffWindowObjects) do
      value:NewTimer(effect)
    end
  end
  -- End Buff Windows Effect Handling

  -- Player effects added
  self.PlayerEffectAdded = function()

    -- A E SE SP
    if Settings.AEnabled and Settings.EEnabled and Settings.SEnabled and Settings.SPEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
    -- A E SE
    elseif Settings.AEnabled and Settings.EEnabled and Settings.SEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- A E SP
    elseif Settings.AEnabled and Settings.EEnabled and Settings.SPEnabled  then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- A SE SP
    elseif Settings.AEnabled and Settings.SEnabled and Settings.SPEnabled  then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- E SE SP
    elseif Settings.EEnabled and Settings.SEnabled and Settings.SPEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
    -- A E
    elseif Settings.AEnabled and Settings.EEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
    -- A SP
    elseif Settings.AEnabled and Settings.SPEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- A SE
    elseif  Settings.AEnabled and Settings.SPEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
    -- E SP
    elseif Settings.EEnabled and Settings.SEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end

        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- E SE
    elseif Settings.EEnabled and Settings.SPEnabled  then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
          -- Player Timers
          self.PlayerTimerHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    -- SP SE
    elseif Settings.SEnabled and Settings.SPEnabled  then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Player Timers
          self.PlayerTimerHandling(effect)
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
      
    elseif Settings.AEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Alerts
          self.PlayerAlertWindowHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    elseif Settings.EEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Effect alerts
          self.PlayerEffectWindowHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    elseif Settings.SEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Player Timers
          self.PlayerTimerHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)

    elseif Settings.SPEnabled then
      local effectHandling = function(sender, args)
        local effect = sender:Get(args.Index)
        if self.SelfEffectsLUT[effect:GetName()] then 
          -- Player Effect Timers
          self.PlayerEffectTimerHandling(effect)
        end
    
        -- Buff bars
        self.PlayerBuffWindowHandling(effect)
      end

      self.callbacks["EffectAdded"] = AddCallback(self.PlayerEffects, "EffectAdded", effectHandling)
    end
  end

  self.PlayerEffectAdded()
end

function PlayerEffectEvents:RemoveCallbacks()
  for key, value in pairs(self.callbacks) do
    RemoveCallback(self.PlayerEffects, key, value)
  end
end

function PlayerEffectEvents:AlterAllEffectsAlerts(key, value)
  self.AllEffectsAdded[key] = value

  if value then self:AlterSelfEffectsLUT(key, value) end
end

function PlayerEffectEvents:AlterEffectsAdded(key, value)
  self.EffectsAdded[key] = value

  if value then self:AlterSelfEffectsLUT(key, value) end
end

function PlayerEffectEvents:AlterTEffects(key, value)
  self.TEffects[key] = value

  if value then self:AlterSelfEffectsLUT(key, value) end
end

function PlayerEffectEvents:AlterAllPlayerEffects(key, value)
  self.AllPlayerEffects[key] = value

  if value then self:AlterSelfEffectsLUT(key, value) end
end

function PlayerEffectEvents:AlterPlayerEffectsTable(key, value)
  self.PlayerEffectsTable[key] = value

  if value then self:AlterSelfEffectsLUT(key, value) end
end

function PlayerEffectEvents:AlterSelfEffectsLUT(key, value)
  self.SelfEffectsLUT[key] = value
end

GroupEffectEvents = class(Turbine.UI.Control)
function GroupEffectEvents:Constructor()
  Turbine.UI.Control.Constructor(self)
  self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()

  -- Information tables
  self.Group = {}
  self.GroupNameLUT = {}
  self.GroupRats = {}

  -- Event tables
  self.DNFstring = DNFstring
  self.AllEffectsAdded = E.AllEffectsAdded
  self.AllPlayerEffects = E.AllPlayerEffects
  self.PlayerEffects = E.PlayerEffects
  self.TAllEffects = TAllEffects
  self.GroupEffectsLUT = GroupEffectsLUT
  self.Player = Player

  self.GroupSize = 0

  self.callbacks = {}

  -- Begin DNF group handling
  self.DNFGroupHandling = function(effect, i)
    if effect:GetName() == self.DNFstring then
      rezTracker:DNFApplied(self.Group[i].player:GetName(), effect:GetStartTime() + effect:GetDuration())
    end
  end
  -- End Dnf Group Handling

  -- Begin Group Effect Handling
  self.GroupEffectHandling = function(effect, i)

    local AlertTemp = self.AllEffectsAdded[effect:GetName()]
    if AlertTemp then
      -- Handle sound effect
      if AlertTemp[5] then Sounds:PlaySoundEffect(AlertTemp[5]) end
      
      -- Handle Icon
      if AlertTemp[3] == true then AlertTemp[3] = effect:GetIcon() end

      -- Publish message
      messageWindow:Message(AlertTemp[1], AlertTemp[2] .. "<rgb=#FF0000>" .. self.Group[i].player:GetName() .. "</rgb>", AlertTemp[3], AlertTemp[4])

      -- Handle Targetbox events
      if AlertTemp[6] and targetBox then
        local icon = nil
        if AlertTemp[7] ~= nil and AlertTemp[7] ~= true then
          icon = AlertTemp[7]
        else
          icon = effect:GetIcon() 
        end
        targetBox:ShowBox(self.Group[i].player:GetName(), icon) 
      end
    end
  end
  -- End Group Effect Handling

  -- Begin Group Effects Timer Handling
  self.GroupEffectTimerHandling = function(effect, i)

    -- Handle group effects
    local AllPlayerEffect = self.AllPlayerEffects[effect:GetName()]
    if AllPlayerEffect then

      -- Handle effect duration for active timer
      local EffectDuration = AllPlayerEffect[4]
      if effect:GetDuration() and EffectDuration and effect:GetDuration() < 999 then
        EffectDuration = effect:GetDuration()
      elseif EffectDuration == nil then
        EffectDuration = 1
      end

      -- Publish Alerts
      if EffectDuration then
        activeTimers:NewTimer(AllPlayerEffect[1], effect:GetName(), AllPlayerEffect[2], AllPlayerEffect[3], EffectDuration, AllPlayerEffect[5], effect:GetIcon(), self.Group[i].player:GetName())
      end
      skillTimers:NewTimer(AllPlayerEffect[1], effect:GetName(), AllPlayerEffect[2], AllPlayerEffect[3], EffectDuration, AllPlayerEffect[5], effect:GetIcon(), self.Group[i].player:GetName())
    end

    -- Handle player specific effects
    if self.PlayerEffects[self.Group[i].player:GetClass()] then
      local ClassSpecificEffect = self.PlayerEffects[self.Group[i].player:GetClass()][effect:GetName()]
      if ClassSpecificEffect then

        -- Handle effect duration for active timer
        local EffectDuration = ClassSpecificEffect[4]
        if effect:GetDuration() and EffectDuration and effect:GetDuration() < 999 then
          EffectDuration = effect:GetDuration()
        elseif EffectDuration == nil then
          EffectDuration = 1
        end

        -- Publish alerts
        if EffectDuration then
          activeTimers:NewTimer(ClassSpecificEffect[1], effect:GetName(), ClassSpecificEffect[2], ClassSpecificEffect[3], EffectDuration, ClassSpecificEffect[5], effect:GetIcon(), self.Group[i].player:GetName())
        end
        skillTimers:NewTimer(ClassSpecificEffect[1], effect:GetName(), ClassSpecificEffect[2], ClassSpecificEffect[3], EffectDuration, ClassSpecificEffect[5], effect:GetIcon(), self.Group[i].player:GetName())
      end
    end
  end
  -- End Group Effects Timer Handling

  -- Begin Timer Handling
  self.GroupTimerHandling = function(effect, i)
    local TimerEffect = self.TAllEffects[effect:GetName()]

    if TimerEffect == nil then return end

    local message = nil
    if TimerEffect[2] == "<name>" then message = self.Group[i].player:GetName() else message = TimerEffect[2] end

    local effectInput = nil
    if TimerEffect[4] then effectInput = effect end

    if TimerEffect then skillCounters:NewTimer(TimerEffect[1], message, TimerEffect[3], effectInput) end
  end
  -- End Timer Handling

  -- Set Group Rat Infor
  self.SetGroupRats = function()
    self.GroupRats = {}
    if Party and Party:GetAssistTargetCount() > 0 then
      for i = 1, Party:GetAssistTargetCount(), 1 do
        self.GroupRats[Party:GetAssistTarget(i):GetName()] = true
      end
    end
  end

  -- Set Group Info
  self.SetGroupInformation = function()
    self.Group = {}
    self.GroupNameLUT = {}
    self.GroupRats = nil
    if Party then
      self.GroupSize = Party:GetMemberCount()
      for i = 1, GroupSize, 1 do
        self.Group[i] = {};
        self.Group[i].player = Party:GetMember(i);
        self.Group[i].effects = self.Group[i].player:GetEffects()
        self.Group[i].currentEffects = {}
        self.GroupNameLUT[Party:GetMember(i):GetName()] = true
      end
    else
      self.Group[1] = {};
      self.Group[1].player = Player;
      self.Group[1].effects = self.Group[1].player:GetEffects()
      self.Group[1].currentEffects = {}
    end

    self.SetGroupRats()

  end

  -- Set Group Triggers
  self.SetGroupReloads = function()
    self.Player.PartyChanged = function()
      ReloadPlugin()
      --ChatOutput("Party changed")
      --if ratTracker ~=  nil then ratTracker:UpdateText() end
    end
    self.Player.RaidChanged = function()
      ReloadPlugin()
      --ChatOutput("Raid changed")
      --if ratTracker ~=  nil then ratTracker:UpdateText() end
    end
    if Party ~= nil then
      Party.MemberAdded = function()
        ReloadPlugin()
        --ChatOutput("Member added")
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
      Party.MemberRemoved = function()
        ReloadPlugin()
        --ChatOutput("Member removed")
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
      Party.LeaderChanged = function()
        ReloadPlugin()
        --if ratTracker ~=  nil then ratTracker:UpdateText() end
      end
  
      for i = 1, Party:GetMemberCount(), 1 do
        Party:GetMember(i).IsLinkDeadChanged = function()
          ReloadPlugin()
          --ChatOutput("Link dead changed")
        end
      end
    end
  end

  -- Refresh group info
  self.GroupEffectsAdded = function()

    -- Ignore all group effects
    if Settings.IDKMode then return end

    -- Go through all players
    for i = 1, Party:GetMemberCount(), 1 do

      -- Return if localplayer -> handled differently
      if self.Group[i].player:GetName() ~= self.Player:GetName() then

        self.callbacks[i] = {}
        -- A SP SE
        if Settings.AEnabled and Settings.SPEnabled and Settings.SEnabled then
          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)

            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- DNF tracker
                self.DNFGroupHandling(effect, i)

                -- Effect alerts
                self.GroupEffectHandling(effect, i)

                -- Skill Timers
                self.GroupEffectTimerHandling(effect, i)

                -- Add timer
                self.GroupTimerHandling(effect, i)
              end
            end
          end)
        -- A SP
        elseif Settings.AEnabled and Settings.SPEnabled then

          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)
            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- DNF tracker
                self.DNFGroupHandling(effect, i)

                -- Effect alerts
                self.GroupEffectHandling(effect, i)

                -- Skill Timers
                self.GroupEffectTimerHandling(effect, i)
              end
            end
          end)

        -- A SE
        elseif Settings.AEnabled and Settings.SEnabled then

          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)
            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- DNF tracker
                self.DNFGroupHandling(effect, i)

                -- Effect alerts
                self.GroupEffectHandling(effect, i)

                -- Add timer
                self.GroupTimerHandling(effect, i)
              end
            end
          end)

        -- SP SE
        elseif Settings.SPEnabled and Settings.SEnabled then
          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)

            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- Skill Timers
                self.GroupEffectTimerHandling(effect, i)

                -- Add timer
                self.GroupTimerHandling(effect, i)
              end
            end
          end)

        -- A
        elseif Settings.AEnabled then
          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)

            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- DNF tracker
                self.DNFGroupHandling(effect, i)

                -- Effect alerts
                self.GroupEffectHandling(effect, i)
              end
            end
          end)

        -- SP
        elseif Settings.SPEnabled then
          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)

            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- Skill Timers
                self.GroupEffectTimerHandling(effect, i)
              end
            end
          end)
        -- SE
        elseif Settings.SEnabled then
          self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)

            local effect = sender:Get(args.Index)
            if not(self.Group[i].currentEffects[effect:GetID()]) then
              self.Group[i].currentEffects[effect:GetID()] = true
              if self.GroupEffectsLUT[effect:GetName()] then 

                -- Add timer
                self.GroupTimerHandling(effect, i)

              end
            end
          end)
        end
      end
    end
  end

  self.RefreshGroupData = function()
    if Party ~= nil then
      for i = 1, Party:GetMemberCount(), 1 do
        self.Group[i].effects = nil
        self.Group[i].currentEffects = nil
      end
      collectgarbage()
      for i = 1, Party:GetMemberCount(), 1 do
        self.Group[i].effects = self.Group[i].player:GetEffects()
        self.Group[i].currentEffects = {}
      end
    end
    self.GroupRats = {}
    if Party ~= nil and Party:GetAssistTargetCount() > 0 then
      for i = 1, Party:GetAssistTargetCount(), 1 do
        self.GroupRats[Party:GetAssistTarget(i):GetName()] = true
      end
    end
  end

  self.SetGroupInformation()
  self.SetGroupReloads()

  -- Party check
  if Party then 
    self.GroupEffectsAdded()
    ChatEventHandler:SetPartyData()
  end
end

function GroupEffectEvents:RemoveCallbacks()
  if Party then
    for i = 1, self.GroupSize, 1 do
      --ChatOutput(self.Group[i].player:GetName())
      if self.Group[i].player:GetName() ~= self.Player:GetName() then 
        for key, value in pairs(self.callbacks[i]) do
          RemoveCallback(self.Group[i].effects, key, value)
        end
      end
    end
  end
end

function GroupEffectEvents:CheckGroupNameLUT(name)
  return(self.GroupNameLUT[name])
end

function GroupEffectEvents:CheckGroupRat(name)
  return(self.GroupRats[name])
end

function GroupEffectEvents:AlterAllEffectsAlerts(key, value)
  self.AllEffectsAdded[key] = value

  if value then self:AlterGroupEffectsLUT(key, value) end
end

function GroupEffectEvents:AlterAllPlayerEffects(key, value)
  self.AllPlayerEffects[key] = value

  if value then self:AlterGroupEffectsLUT(key, value) end
end

function GroupEffectEvents:AlterPlayerEffects(key, value)
  self.PlayerEffects[key] = value

  if value then self:AlterGroupEffectsLUT(key, value) end
end

function GroupEffectEvents:AlterTEffects(key, value)
  self.TAllEffects[key] = value

  if value then self:AlterGroupEffectsLUT(key, value) end
end

function GroupEffectEvents:AlterGroupEffectsLUT(key, value)
  self.GroupEffectsLUT[key] = value
end
    -- Self only effects handling -- deprecated
    --[[
    local SelfEffect = E.SelfOnlyEffects[effect:GetName()]
    if SelfEffect ~= nil then
      local activityTimer1 = SelfEffect[4]
      if effect:GetDuration() ~= nil  and activityTimer1 ~= nil and effect:GetDuration() < 999 then
        activityTimer1 = effect:GetDuration()
      elseif activityTimer1 == nil then
        activityTimer1 = 1
      end
      if activityTimer1 ~= nil then
        activeTimers:NewTimer(SelfEffect[1], effect:GetName(), SelfEffect[2], SelfEffect[3], activityTimer1, SelfEffect[5], effect:GetIcon(), Player:GetName())
      end
      skillTimers:NewTimer(SelfEffect[1], effect:GetName(), SelfEffect[2], SelfEffect[3], activityTimer1, SelfEffect[5], effect:GetIcon(), Player:GetName())
    end
      --]]